@@ -1,7 +1,7 @@
 __name__ = "chkpkg",
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 __author__ = "Artёm IG"
 __author_email__ = "ortemeo@gmail.com"
 __license__ = "MIT"
-__summary__ = "Checks Python packages intended to be published on PyPi",
+__summary__ = "Checks Python packages intended to be published on PyPi"
 __keywords__ = "package distribution wheel whl testing".split()
